#include "common.h"
#include "thread_main.h"
#include "gpio_pwm_carship.h"
#include "gpio_pwm_ptz.h"
#include "mqtt.h"
#include "judg.h"
#include "delay.h"
#include "car.h"
#include "tank.h"
#include "ship.h"
#include "pao.h"
#include "fileopen.h"
#include "gpio_pwm_tank0202.h"


char TOPIC_middle[23];
char TOPIC2_middle[23];
char TOPIC3_middle[15];

void topic_middle_value() {
    TOPIC=TOPIC_middle;
    TOPIC2=TOPIC2_middle;
    TOPIC3=TOPIC3_middle;
}

// 哈希函数：简单的字符串哈希
unsigned int hash(const char *str, int tableSize) {
    unsigned int hash = 0;
    while (*str) {
        hash = (hash << 5) + *str++;
    }
    return hash % tableSize;
}

// 创建哈希表
HashTable* createHashTable(int size) {
    HashTable *hashTable = (HashTable *)malloc(sizeof(HashTable));
    hashTable->size = size;
    hashTable->table = (HashEntry **)malloc(sizeof(HashEntry *) * size);
    for (int i = 0; i < size; i++) {
        hashTable->table[i] = NULL;
    }
    return hashTable;
}

// 插入元素到哈希表
void insert(HashTable *hashTable, const char *key, CodeEnum value) {
    unsigned int index = hash(key, hashTable->size);
    HashEntry *newEntry = (HashEntry *)malloc(sizeof(HashEntry));
    newEntry->key = key;
    newEntry->value = value;
    newEntry->next = hashTable->table[index];
    hashTable->table[index] = newEntry;
}

// 查找元素
CodeEnum find(HashTable *hashTable, const char *key) {
    unsigned int index = hash(key, hashTable->size);
    HashEntry *entry = hashTable->table[index];
    while (entry) {
        if (strcmp(entry->key, key) == 0) {
            return entry->value;
        }
        entry = entry->next;
    }
    return -1;  // 未找到
}

int hash_insert_init(HashTable *hashTable) {

    // 插入已有的代码
    insert(hashTable, "0101", CAR_0101);
    insert(hashTable, "0102", CAR_0102);
    insert(hashTable, "0201", TANK_0201);
    insert(hashTable, "0202", TANK_0202);  
    insert(hashTable, "0301", SHIP_0301);
    insert(hashTable, "0401", PAO_0401);
}

int Device_judg(CodeEnum code,char *sub_str) {
    if (code == CAR_0101) {
        car_Init_0101() ;
        my_zlog_info("使用型号%s",sub_str);
    }else if(code == CAR_0102) {
        car_Init_0102();
        my_zlog_info("使用型号%s",sub_str);
    }else if(code == TANK_0201) {
        tank_Init_0201();
        my_zlog_info("使用型号%s",sub_str);
    }else if(code == TANK_0202) {
        tank_Init_0202();
        my_zlog_info("使用型号%s",sub_str);
    }else if(code == SHIP_0301) {
        ship_Init_0301();
        my_zlog_info("使用型号%s",sub_str);
    }else if(code ==PAO_0401) {
        ptr_Init_0401();
        my_zlog_info("使用型号%s",sub_str);   
    }else {
        my_zlog_error("没有找到设备号");
        exit(0);
    }
    return 0;
}

int Device_File_Init() {
    const char *readbuf=device_inspect(); 

    //将第3个和第4个字符提取出来
    char *sub_str=malloc(5);
    sub_str[0]=readbuf[2];
    sub_str[1]=readbuf[3];
    sub_str[2]=readbuf[4];
    sub_str[3]=readbuf[5];
    sub_str[4]='\0';
    my_zlog_info("开始初始化了");

    HashTable *hashTable = createHashTable(INITIAL_TABLE_SIZE);
    hash_insert_init(hashTable);
    CodeEnum code = find(hashTable, sub_str);
    
    if(Device_judg(code,sub_str) == 0) my_zlog_info("判断成功了");

    free(sub_str);
          
    Delay_Ms(20,0);

    sprintf(TOPIC2_middle,"dev2app/%s",readbuf);
    sprintf(TOPIC_middle,"app2dev/%s",readbuf);
    sprintf(TOPIC3_middle,"%s",readbuf);

    topic_middle_value();//指针传值声明

    my_zlog_info("1:%s,2:%s,3:%s",TOPIC,TOPIC2,TOPIC3);

    return 0;
}
